'use strict';
/**
 * @class 1ポリシーに対する操作をするためのクラス
 * @property {string} _isInvalid このポリシー内に一つでもバリデーションに引っかかる項目があるかのフラグ
 * @property {string} table_data テーブル用のHTMLデータ
 * @property {string} url このポリシー試験用のurlデータ
 */
var Policy = function () {
    this.isInvalid = false;
    this.table_data = "";
    this.request_url = "";
    //初期化
    this.protocol = new Protocol();
    this.src_fw_ip = new SourceFWIP();
    this.dst_fw_ip = new DestinationFWIP();
    this.src_ip = new SourceIP();
    this.src_mask = new SourceMask();
    this.src_port = new SourcePort();
    this.dst_ip = new DestinationIP();
    this.dst_mask = new DestinationMask();
    this.dst_port = new DestinationPort();
    this.nat_dst_ip = new DestinationNatIP();
    this.nat_dst_port = new DestinationNatPort();
    this.expected_result = new ExpectedResult();

    // Source NAT
    this.nat_src_ip = new SourceNatIP();


    this.field = {
        "protocol": this.protocol,
        "src_fw_ip": this.src_fw_ip,
        "src_mask": this.src_mask,
        "src_ip": this.src_ip,
        "src_port": this.src_port,
        "nat_src_ip": this.nat_src_ip,
        "dst_fw_ip": this.dst_fw_ip,
        "dst_ip": this.dst_ip,
        "dst_port": this.dst_port,
        "nat_dst_ip": this.nat_dst_ip,
        "dst_mask": this.dst_mask,
        "nat_dst_port": this.nat_dst_port,
        "expected_result": this.expected_result,
        "result": "",
        "isMismatch":false,
        "description": ""
    };
    /**
     * テーブルデータを取得する関数
     * @ return {string} テーブルデータを返す
     */
    Policy.prototype.getTableData = function () {
        return this.table_data;
    }
    /**
     * 含まれるエラーを返す関数
     * @param {number} policy_num 試験番号
     * @return {string} 複数のエラーメッセージ
     */
    Policy.prototype.getMessage = function (policy_num) {
        let that = this;
        let error_msg = "";
        let field_error_msg = "";
        if (typeof policy_num === "undefined") {//oneline
            $.each(this.field, function (key, element) {
                if (key === "result") return false;//これ以降はオブジェクトではないのでgetMessageは無い。
                if (that.field[key].isInvalid) {
                    error_msg += that.field[key].getMessage();
                }
            });
        } else {//scenario
            $.each(this.field, function (key, element) {
                if (key === "result") return false;//これ以降はオブジェクトではないのでgetMessageは無い。
                if (that.field[key].isInvalid) {
                  field_error_msg = that.field[key].getMessage();
                  if (field_error_msg.match(/<br>+/)) {
                    //1つのフィールドに複数のエラーメッセージが含まれている場合、エラーメッセージの前にポリシー番号を付与する
                    error_msg += policy_num + ": " + field_error_msg.replace(/<br>/g, "\n" + policy_num + ": ");
                    error_msg = error_msg.replace(/\d+:\s$/, ""); // 行末のポリシー番号を削除する。
                  } else {
                    error_msg += policy_num + ": " + field_error_msg;
                  }
                }
            });
        }
        return error_msg;
    }
    /**
     * 期待する試験結果を変えす関数
     * @return {string} 期待する試験結果
     */
    Policy.prototype.getExpectResult = function () {
        return this.field["expected_result"].value;
    }
    /**
     * 各項目にcsvから読み込んだデータを設定する関数
     * @param {array} csv_data csvをファイルリーダで読み込んだ後の配列データ
     */
    Policy.prototype.setCSVData = function (csv_data) {
        this.field.protocol.setValue(csv_data[0]);
        this.field.src_fw_ip.setValue(csv_data[1]);
        this.field.dst_fw_ip.setValue(csv_data[2]);
        this.field.src_ip.setValue(csv_data[3]);
        this.field.src_mask.setValue(csv_data[4]);
        this.field.src_port.setValue(csv_data[5]);
        this.field.nat_src_ip.setValue(csv_data[6]);
        this.field.dst_ip.setValue(csv_data[7]);
        this.field.dst_mask.setValue(csv_data[8]);
        this.field.dst_port.setValue(csv_data[9]);
        this.field.nat_dst_ip.setValue(csv_data[10]);
        this.field.nat_dst_port.setValue(csv_data[11]);
        this.field.expected_result.setValue(csv_data[12]);
        this.field.description = csv_data[13];
    }
    /**
     * 各項目にDOMから読み込んだデータを設定する関数
     */
    Policy.prototype.setDOMData = function () {
        //console.info("*Setting Value...");
        let that = this;
        $.each(this.field, function (key, element) {
        //console.log(this.field[key])
            if (key === "expected_result") return false;//これ以降はオブジェクトではないので関数がない
            that.field[key].setFromDOM();
        });
        //console.info("Done")
    }
    /**
     * 各項目のバリデーションを実行する関数
     */
    Policy.prototype.Validate = function () {
        //console.log(this.field)
        let that = this;
        if (this.field["protocol"].isICMP) {//ICMPの時
            console.info("**ICMP Validation...");
            this.field["src_port"].ValidateICMP();
            this.field["dst_port"].ValidateICMP();
            this.field["nat_dst_port"].ValidateICMP();
            $.each(this.field, function (key, element) {
                if (key === "result") return false;//これ以降はオブジェクトではないのでvalidateは無い。
                if (!that.isInvalid) {//一つでもエラーがあれば無視する。
                    if (that.field[key].isInvalid) {
                        that.isInvalid = true;
                        return false;
                    }
                }
            });
            console.info("Done");
        } else if (this.field["protocol"].isTCP || this.field["protocol"].isUDP) {//TCP/UDPの時
            console.info("**TCP/UDP Validation...");
            this.field["src_port"].ValidateTCPUDP();
            this.field["dst_port"].ValidateTCPUDP();
            this.field["nat_dst_port"].ValidateTCPUDP();
            //nat dst ipがるかどうかでvalidationの内容が変化する。
            if (this.field["nat_dst_ip"].isEmpty) {
                this.field["nat_dst_port"].ValidateTCPUDPNat();
            } else {
                this.field["nat_dst_port"].ValidateTCPUDP();
            }
            $.each(this.field, function (key, element) {
                if (key === "result") return false;//これ以降はオブジェクトではないのでvalidateは無い。
                if (!that.isInvalid) {
                    if (that.field[key].isInvalid) {
                        that.isInvalid = true;
                        return false;
                    }
                }
            });
            console.info("Done");
        }
        CommonValid();
        /**
         * ICMP TCP/UDPで共通のvalidationを行う関数
         * @inner
        */
        function CommonValid() {
            that.field["protocol"].Validate();
            that.field["src_fw_ip"].Validate();
            that.field["dst_fw_ip"].Validate();
            that.field["src_ip"].ValidateScenario();
            that.field["src_mask"].Validate();
            that.field["dst_ip"].ValidateScenario();
            that.field["dst_mask"].Validate();
            that.field["nat_src_ip"].Validate();
            that.field["nat_dst_ip"].Validate();
            if(that["test_type"] !== "oneline") {
              that.field["expected_result"].ValidateScenario();
            }
            CheckDuplicateIP();
        }
        /**
         * IP重複に関するvalidationを行う関数
         * @inner
        */
        function CheckDuplicateIP() {
            var verify = 0;
            if (that.field["src_fw_ip"].value !== "") {
                if (that.field["src_fw_ip"].value === that.field["dst_fw_ip"].value ){
                    that.field["src_fw_ip"].message += ERROR_MESSAGES["CheckDuplicateIP"].src_fw_ip + " and " + ERROR_MESSAGES["CheckDuplicateIP"].dst_fw_ip + ERROR_MESSAGES["CheckDuplicateIP"].Remaining + "<br>";
                    verify = 1;
                }
                if (that.field["src_fw_ip"].value === that.field["src_ip"].value ){
                    that.field["src_fw_ip"].message += ERROR_MESSAGES["CheckDuplicateIP"].src_fw_ip + " and " + ERROR_MESSAGES["CheckDuplicateIP"].src_ip + ERROR_MESSAGES["CheckDuplicateIP"].Remaining + "<br>";
                    verify = 1;
                }
                if (that.field["src_fw_ip"].value === that.field["dst_ip"].value ){
                    that.field["src_fw_ip"].message += ERROR_MESSAGES["CheckDuplicateIP"].src_fw_ip + " and " + ERROR_MESSAGES["CheckDuplicateIP"].dst_ip + ERROR_MESSAGES["CheckDuplicateIP"].Remaining + "<br>";
                    verify = 1;
                }
                if (verify == 1) {
                    that.src_fw_ip.isInvalid = true;
                    ++ verify;
                }
            }
            if (that.field["dst_fw_ip"].value !== "") {
                if (that.field["dst_fw_ip"].value === that.field["src_ip"].value ){
                    that.field["dst_fw_ip"].message += ERROR_MESSAGES["CheckDuplicateIP"].dst_fw_ip + " and " + ERROR_MESSAGES["CheckDuplicateIP"].src_ip + ERROR_MESSAGES["CheckDuplicateIP"].Remaining + "<br>";
                    verify = 1;
                }
                if (that.field["dst_fw_ip"].value === that.field["dst_ip"].value ){
                    that.field["dst_fw_ip"].message += ERROR_MESSAGES["CheckDuplicateIP"].dst_fw_ip + " and " + ERROR_MESSAGES["CheckDuplicateIP"].dst_ip + ERROR_MESSAGES["CheckDuplicateIP"].Remaining + "<br>";
                    verify = 1;
                }
                if (verify == 1) {
                    that.dst_fw_ip.isInvalid = true;
                    ++ verify;
                }
            }
            if (that.field["src_ip"].value !== "") {
                if (that.field["src_ip"].value === that.field["dst_ip"].value ){
                    that.field["src_ip"].message += ERROR_MESSAGES["CheckDuplicateIP"].src_ip + " and " + ERROR_MESSAGES["CheckDuplicateIP"].dst_ip + ERROR_MESSAGES["CheckDuplicateIP"].Remaining + "<br>";
                    verify = 1;
                }
                if (that.field["src_ip"].value === that.field["nat_dst_ip"].value ){
                    that.field["src_ip"].message += ERROR_MESSAGES["CheckDuplicateIP"].src_ip + " and " + ERROR_MESSAGES["CheckDuplicateIP"].nat_dst_ip + ERROR_MESSAGES["CheckDuplicateIP"].Remaining + "<br>";
                    verify = 1;
                }
                if (verify == 1) {
                    that.src_ip.isInvalid = true;
                    ++ verify;
                }
            }
            if (that.field["dst_ip"].value !== "") {
                if (that.field["dst_ip"].value === that.field["nat_dst_ip"].value ){
                    that.field["dst_ip"].message += ERROR_MESSAGES["CheckDuplicateIP"].dst_ip + " and " + ERROR_MESSAGES["CheckDuplicateIP"].nat_dst_ip + ERROR_MESSAGES["CheckDuplicateIP"].Remaining + "<br>";
                    verify = 1;
                }
                if (verify == 1) {
                    that.dst_ip.isInvalid = true;
                    ++ verify;
                }
            }

            if (verify === 2) {
                console.log("verify:" + verify);
                return;
            }
        }
    }

    /**
     * テーブルへ挿入するHTMLデータを組み立てる関数
     * @param {number} policy_num ポリシーの番号
     */
    Policy.prototype.BuildTableRow = function (policy_num) {
        let table_data = "";
        table_data += '<tr class="scenario-table-row disp-tbl-row" id="scenario-table-row' + policy_num + '">'
        table_data += '<td>' + (policy_num + 1) + '</td>';
        table_data += '<td>' + this.field["protocol"].value + '</td>'; // protocol
        table_data += '<td>' + this.field["src_fw_ip"].value + '</td>'; // src-side fw ip
        table_data += '<td>' + this.field["dst_fw_ip"].value + '</td>'; // dst-side fw ip
        table_data += '<td>' + concat(this.field["src_ip"].value, this.field["src_mask"].value, this.field["src_port"].value) + '</td>'; //src-host
        table_data += '<td>' + this.field["nat_src_ip"].value + '</td>';//nat-src
        table_data += '<td>' + concat(this.field["dst_ip"].value, this.field["dst_mask"].value, this.field["dst_port"].value) + '</td>'; //dst-host
        table_data += '<td>' + concat(this.field["nat_dst_ip"].value, '', this.field["nat_dst_port"].value) + '</td>';//nat-dst
        table_data += '<td class="expected">' + this.field["expected_result"].value + '</td>'; // result(expect)
        table_data += '<td class="result">-</td>'; // result
        table_data += '<td class="send"><button type="button" class="btn btn-default btn-xs scenario-send-btn" onClick="scenario.SendOne(' + policy_num + ')"><i class="fa fa-play"></i></button> </td>';  // retry
        table_data += '<td class="time">-</td>'; // time
        table_data += '<td>' + this.field["description"] + '</td>'; // description
        table_data += '</tr>\n';
        this.table_data = table_data;
        /**
         * 送信用データの一部を結合する関数
         * @inner
         */
        function concat(ip, mask, port) {
            let url = '';
            url += ip;
            if (mask !== '') {
                url += '/' + mask;
            }
            if (port !== '') {
                url += ':' + port;
            }
            return url;
        }
    }
    /**
     * 各項目から送信用のURlを組み立てる関数
     */
    Policy.prototype.BuildURL = function () {
        let url = 'http://192.0.2.1:8080/1/policy?'
        url += 'p=' + this.field["protocol"].value;
        url += '&sfw=' + this.field["src_fw_ip"].value;
        url += '&dfw=' + this.field["dst_fw_ip"].value;
        url += '&src=' + concat(this.field["src_ip"].value, this.field["src_mask"].value, this.field["src_port"].value);
        if (!this.field["nat_src_ip"].isEmpty) {
            url += '-' + this.field["nat_src_ip"].value;
        }
        url += '&dst=' + concat(this.field["dst_ip"].value, this.field["dst_mask"].value, this.field["dst_port"].value);
        if (!this.field["nat_dst_ip"].isEmpty) {
            url += '-' + this.field["nat_dst_ip"].value;
        }
        if (!this.field["nat_dst_port"].isEmpty) {
            url += ':' + this.field["nat_dst_port"].value;
        }
        this.request_url = url;
        /**
        * 送信用データの一部を結合する関数
        * @inner
        */
        function concat(ip, mask, port) {
            let url = '';
            url += ip;
            if (mask !== '') {
                url += '/' + mask;
            }
            if (port !== '') {
                url += ':' + port;
            }
            return url;
        }
    }
    /**
     * データを送信する関数
     * @inner
     */
    Policy.prototype.Send = function () {
        return $.ajax({
            type: 'GET',
            url: this.request_url,
            dataType: 'json',
            timeout: 30000,
        })
    }
    /**
     * Mismatch判定を変更する関数
     * @ param {bool} bool セットするmismatch
     */
    Policy.prototype.setMismatch = function (bool) {
        if(typeof bool === "boolean"){
            this.field["isMismatch"] = bool;
        }else{
            this.field["isMismatch"] = bool;
        }
    }
 }
